@@ -44,7 +44,7 @@
 
 static void     gpm_prefs_class_init (GpmPrefsClass *klass);
 static void     gpm_prefs_init       (GpmPrefs      *prefs);
-static void     gpm_prefs_finalize   (GObject	  *object);
+static void     gpm_prefs_finalize   (GObject	    *object);
 
 #define GPM_PREFS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_PREFS, GpmPrefsPrivate))
 
@@ -81,6 +81,7 @@ static GConfEnumStringPair icon_policy_enum_map [] = {
 };
 
 /* The text that should appear in the action combo boxes */
+#define ACTION_INTERACTIVE_TEXT		_("Ask me")
 #define ACTION_SUSPEND_TEXT		_("Suspend")
 #define ACTION_SHUTDOWN_TEXT		_("Shutdown")
 #define ACTION_HIBERNATE_TEXT		_("Hibernate")
@@ -127,7 +128,7 @@ gpm_prefs_class_init (GpmPrefsClass *klass)
 
 /**
  * gpm_dbus_method_bool:
- * @method: The g-p-m DBUS method name, e.g. "CanSuspend"
+ * @method: The g-p-m DBUS method name, e.g. "AllowedSuspend"
  **/
 static gboolean
 gpm_dbus_method_bool (const char *method)
@@ -401,6 +402,8 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 		action = ACTION_BLANK;
 	} else if (strcmp (value, ACTION_NOTHING_TEXT) == 0) {
 		action = ACTION_NOTHING;
+	} else if (strcmp (value, ACTION_INTERACTIVE_TEXT) == 0) {
+		action = ACTION_INTERACTIVE;
 	} else {
 		g_assert (FALSE);
 	}
@@ -466,6 +469,10 @@ gpm_prefs_setup_action_combo (GpmPrefs    *prefs,
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_BLANK_TEXT);
 			n_added++;
+		} else if (strcmp (actions[i], ACTION_INTERACTIVE) == 0) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+						   ACTION_INTERACTIVE_TEXT);
+			n_added++;
 		} else if (strcmp (actions[i], ACTION_NOTHING) == 0) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_NOTHING_TEXT);
@@ -934,8 +941,8 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value ("button",
 							"button.type",
 							"lid") > 0;
-	prefs->priv->can_suspend = gpm_dbus_method_bool ("CanSuspend");
-	prefs->priv->can_hibernate = gpm_dbus_method_bool ("CanHibernate");
+	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
+	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
 
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
 
@@ -966,6 +973,17 @@ gpm_prefs_init (GpmPrefs *prefs)
 	setup_ups_actions (prefs);
 	setup_sleep_type (prefs);
 
+	const char   *power_button_actions[] = {ACTION_INTERACTIVE,
+						ACTION_SUSPEND,
+						ACTION_HIBERNATE,
+						ACTION_SHUTDOWN,
+						NULL};
+
+	/* Power button action */
+	gpm_prefs_setup_action_combo (prefs, "combobox_power_button",
+				      GPM_PREF_BUTTON_POWER,
+				      power_button_actions);
+
 	int delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
 	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
